@@ -7,6 +7,7 @@ import fcntl
 import grp
 import os
 import stat
+import sys
 import tempfile
 import unittest
 
@@ -18,8 +19,8 @@ except ImportError:
 from snakeoil import osutils
 from snakeoil.fileutils import touch
 from snakeoil.test import TestCase, SkipTest, mk_cpy_loadable_testcase
-from snakeoil.osutils import native_readdir
-from snakeoil.osutils.mount import mount, umount, MS_BIND
+from snakeoil.osutils import native_readdir, supported_systems
+from snakeoil.osutils.mount import mount, umount, MNT_FORCE, MS_BIND
 from snakeoil.test.mixins import TempDirMixin
 
 pjoin = os.path.join
@@ -311,57 +312,98 @@ class FsLockTest(TempDirMixin):
         self.assertRaises(osutils.NonExistent, osutils.FsLock,
                           pjoin(self.dir, 'missing'))
 
-    def test_locking(self):
-        path = pjoin(self.dir, 'lockfile')
+    def test_fslock_read_lock(self):
+        path = pjoin(self.dir, 'lockfile-read')
         lock = osutils.FsLock(path, True)
         # do this all non-blocking to avoid hanging tests
         self.assertTrue(lock.acquire_read_lock(False))
         # file should exist now
-        with open(path) as f:
+        with open(path, 'r+') as f:
             # acquire and release a read lock
             fcntl.flock(f, fcntl.LOCK_SH | fcntl.LOCK_NB)
-            fcntl.flock(f, fcntl.LOCK_UN | fcntl.LOCK_NB)
+            fcntl.flock(f, fcntl.LOCK_UN)
             # we can't acquire an exclusive lock
             self.assertRaises(
                 IOError, fcntl.flock, f, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            lock.release_read_lock()
+
+    def test_fslock_release_read_lock(self):
+        path = pjoin(self.dir, 'lockfile-release-read')
+        lock = osutils.FsLock(path, True)
+        # do this all non-blocking to avoid hanging tests
+        self.assertTrue(lock.acquire_read_lock(False))
+        lock.release_read_lock()
+        # file should exist now
+        with open(path, 'r+') as f:
             # but now we can
             fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            self.assertFalse(lock.acquire_read_lock(False))
-            self.assertFalse(lock.acquire_write_lock(False))
-            fcntl.flock(f, fcntl.LOCK_UN | fcntl.LOCK_NB)
-            # acquire an exclusive/write lock
-            self.assertTrue(lock.acquire_write_lock(False))
-            self.assertRaises(
-                IOError, fcntl.flock, f, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            # downgrade to read lock
-            self.assertTrue(lock.acquire_read_lock())
-            fcntl.flock(f, fcntl.LOCK_SH | fcntl.LOCK_NB)
-            fcntl.flock(f, fcntl.LOCK_UN | fcntl.LOCK_NB)
-            self.assertRaises(
-                IOError, fcntl.flock, f, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            # and release
-            lock.release_read_lock()
-            fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            fcntl.flock(f, fcntl.LOCK_UN | fcntl.LOCK_NB)
+            fcntl.flock(f, fcntl.LOCK_UN)
 
-            self.assertTrue(lock.acquire_write_lock(False))
-            lock.release_write_lock()
+    def test_fslock_write_lock(self):
+        path = pjoin(self.dir, 'lockfile-write')
+        lock = osutils.FsLock(path, True)
+        # do this all non-blocking to avoid hanging tests
+        # acquire an exclusive/write lock
+        self.assertTrue(lock.acquire_write_lock(False))
+        # file should exist now
+        with open(path, 'r+') as f:
+            self.assertRaises(
+                IOError, fcntl.flock, f, fcntl.LOCK_EX | fcntl.LOCK_NB)
+
+    def test_fslock_release_write_lock(self):
+        path = pjoin(self.dir, 'lockfile-release-write')
+        lock = osutils.FsLock(path, True)
+        # do this all non-blocking to avoid hanging tests
+        # acquire an exclusive/write lock
+        self.assertTrue(lock.acquire_write_lock(False))
+        lock.release_write_lock()
+        # file should exist now
+        with open(path, 'r+') as f:
             fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
-            fcntl.flock(f, fcntl.LOCK_UN | fcntl.LOCK_NB)
+            fcntl.flock(f, fcntl.LOCK_UN)
+
+    def test_fslock_downgrade_lock(self):
+        path = pjoin(self.dir, 'lockfile-downgrade')
+        lock = osutils.FsLock(path, True)
+        # do this all non-blocking to avoid hanging tests
+        # acquire an exclusive/write lock
+        self.assertTrue(lock.acquire_write_lock(False))
+        # downgrade to read lock
+        self.assertTrue(lock.acquire_read_lock())
+        # file should exist now
+        with open(path, 'r+') as f:
+            fcntl.flock(f, fcntl.LOCK_SH | fcntl.LOCK_NB)
+            fcntl.flock(f, fcntl.LOCK_UN)
+            self.assertRaises(
+                IOError, fcntl.flock, f, fcntl.LOCK_EX | fcntl.LOCK_NB)
+
+    def test_fslock_release_downgraded_lock(self):
+        path = pjoin(self.dir, 'lockfile-release-downgraded')
+        lock = osutils.FsLock(path, True)
+        # do this all non-blocking to avoid hanging tests
+        # acquire an exclusive/write lock
+        self.assertTrue(lock.acquire_write_lock(False))
+        # downgrade to read lock
+        self.assertTrue(lock.acquire_read_lock())
+        # and release
+        lock.release_read_lock()
+        # file should exist now
+        with open(path, 'r+') as f:
+            fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            fcntl.flock(f, fcntl.LOCK_UN)
 
 
 class TestAccess(TempDirMixin):
 
-    if osutils.access is os.access:
-        skip = "os.access is usable, no need to test"
-    elif os.getuid() != 0:
+    if os.getuid() != 0:
         skip = "these tests must be ran as root"
 
-    func = staticmethod(osutils.access)
+    func = staticmethod(osutils.fallback_access)
 
     def test_fallback(self):
         fp = pjoin(self.dir, "file")
+        # create the file
+        with open(fp, 'w') as f:
+            pass
         os.chmod(fp, 000)
         self.assertFalse(self.func(fp, os.X_OK))
         self.assertTrue(self.func(fp, os.W_OK))
@@ -385,6 +427,46 @@ class Test_unlink_if_exists(TempDirMixin):
         f(path)
 
 
+class SupportedSystems(unittest.TestCase):
+
+    def test_supported_system(self):
+        @supported_systems('supported')
+        def func():
+            return True
+
+        with mock.patch('snakeoil.osutils.sys') as _sys:
+            _sys.configure_mock(platform='supported')
+            self.assertTrue(func())
+
+    def test_unsupported_system(self):
+        @supported_systems('unsupported')
+        def func():
+            return True
+
+        with self.assertRaises(NotImplementedError):
+            func()
+
+        # make sure we're iterating through the system params correctly
+        with mock.patch('snakeoil.osutils.sys') as _sys:
+            _sys.configure_mock(platform='u')
+            with self.assertRaises(NotImplementedError):
+                func()
+
+    def test_multiple_systems(self):
+        @supported_systems('darwin', 'linux')
+        def func():
+            return True
+
+        with mock.patch('snakeoil.osutils.sys') as _sys:
+            _sys.configure_mock(platform='nonexistent')
+            with self.assertRaises(NotImplementedError):
+                func()
+
+            for platform in ('linux2', 'darwin'):
+                _sys.configure_mock(platform=platform)
+                self.assertTrue(func())
+
+
 class Mount(unittest.TestCase):
 
     def setUp(self):
@@ -395,6 +477,7 @@ class Mount(unittest.TestCase):
         os.rmdir(self.source)
         os.rmdir(self.target)
 
+    @unittest.skipUnless(sys.platform.startswith('linux'), 'supported on Linux only')
     def test_args_bytes(self):
         # The initial source, target, and fstype arguments to mount(2) must be
         # byte strings; if they are unicode strings the arguments get mangled
@@ -409,12 +492,14 @@ class Mount(unittest.TestCase):
                 for arg in mount_call[1][0:3]:
                     self.assertIsInstance(arg, bytes)
 
+    @unittest.skipUnless(sys.platform.startswith('linux'), 'supported on Linux only')
     def test_missing_dirs(self):
         with self.assertRaises(OSError) as cm:
             mount('source', 'target', None, MS_BIND)
         self.assertEqual(cm.exception.errno, errno.ENOENT)
 
     @unittest.skipIf(os.getuid() == 0, 'this test must be run as non-root')
+    @unittest.skipUnless(sys.platform.startswith('linux'), 'supported on Linux only')
     def test_no_perms(self):
         with self.assertRaises(OSError) as cm:
             mount(self.source, self.target, None, MS_BIND)
@@ -424,6 +509,7 @@ class Mount(unittest.TestCase):
         self.assertTrue(cm.exception.errno in (errno.EPERM, errno.EINVAL))
 
     @unittest.skipIf(os.getuid() != 0, 'this test must be run as root')
+    @unittest.skipUnless(sys.platform.startswith('linux'), 'supported on Linux only')
     def test_root(self):
         # test umount
         mount(self.source, self.target, None, MS_BIND)
